# Generated by roxygen2: do not edit by hand

export(collapse_lattice)
export(count_matches)
export(detect_in_window)
export(detect_subpattern)
export(impute_missing_visits)
export(measure_abstinence_period)
export(measure_retention)
export(recode_missing_visits)
export(view_by_lattice)
export(weight_positive_visits)
importFrom(stringi,stri_locate_last_fixed)
importFrom(stringr,fixed)
importFrom(stringr,str_count)
importFrom(stringr,str_detect)
importFrom(stringr,str_length)
importFrom(stringr,str_replace_all)
importFrom(stringr,str_split)
importFrom(stringr,str_sub)
